/*****************************************
 * Ngram Analysis Tool
 * Author: Robert Hoehndorf <rhoehndo@imn.htwk-leipzig.de>
 * License: GNU Public License
 * Version: 0.15
 * Release Date: 2001-Feb-21
 * Compilation Instructions: A simple 'make' should work fine
 * Usage Instructions: nganal <file1> <file2> <n>
 *   where file1 and file2 are two text-files you would like to
 *   compare, and n is the length of the ngram. This implementation
 *   of the NGram-Analysis need exponentional time and memory, so
 *   the analysis takes O(27^n) time and memory (in the next version
 *   I will optimize a bit, but the Worst Case time and memory will
 *   always be O(27^n)).
 *****************************************/

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include "ngram.h"

int mfgets(char* t,int n,FILE* fp);

int getCharVal(char ch) {
  switch (ch) {
  case 'a':case 'A':return 0;break;
  case 'b':case 'B':return 1;break;
  case 'c':case 'C':return 2;break;
  case 'd':case 'D':return 3;break;
  case 'e':case 'E':return 4;break;
  case 'f':case 'F':return 5;break;
  case 'g':case 'G':return 6;break;
  case 'h':case 'H':return 7;break;
  case 'i':case 'I':return 8;break;
  case 'j':case 'J':return 9;break;
  case 'k':case 'K':return 10;break;
  case 'l':case 'L':return 11;break;
  case 'm':case 'M':return 12;break;
  case 'n':case 'N':return 13;break;
  case 'o':case 'O':return 14;break;
  case 'p':case 'P':return 15;break;
  case 'q':case 'Q':return 16;break;
  case 'r':case 'R':return 17;break;
  case 's':case 'S':return 18;break;
  case 't':case 'T':return 19;break;
  case 'u':case 'U':return 20;break;
  case 'v':case 'V':return 21;break;
  case 'w':case 'W':return 22;break;
  case 'x':case 'X':return 23;break;
  case 'y':case 'Y':return 24;break;
  case 'z':case 'Z':return 25;break;
  case ' ':case '\n':return 26;break;
  default:return 0;
  }
}

long int pow2(int a, int b) {
  int i;
  long res=1;
  for (i=0;i<b;i++)
    res*=a;
  return res;
}

void buildNgram(char* fn1,char* fn2,int n) {
  FILE* fp;
  int i=0;int j;
  int k;
  char* ng;
  int sum;
  ngram1=malloc(pow2(27,n)*sizeof(int));
  ngram2=malloc(pow2(27,n)*sizeof(int));

  ng=malloc(sizeof(char)*(n+1));
  for (i=0;i<pow2(27,n);ngram1[i++]=0);
  for (i=0;i<pow2(27,n);ngram2[i++]=0);

  fp=fopen(fn1,"r");
  fseek(fp,0,SEEK_END);
  j=ftell(fp)-n-1;
  for (i=0;i<j;i++) {
    fseek(fp,i,SEEK_SET);
    mfgets(ng,n,fp);
    for (sum=0,k=0;k<n;k++)
      sum+=getCharVal(ng[k])*pow2(27,k);
    ngram1[sum]++;
  }
  fclose(fp);

  fp=fopen(fn2,"r");
  fseek(fp,0,SEEK_END);
  j=ftell(fp)-n-1;  
  for (i=0;i<j;i++) {
    fseek(fp,i,SEEK_SET);
    mfgets(ng,n,fp);
    for (k=0,sum=0;k<n;k++)
      sum+=getCharVal(ng[k])*pow2(27,k);
    ngram2[sum]++;
  }
  fclose(fp);
}

int mfgets(char* t,int n,FILE* fp) {
  char ch;
  char b=0;
  int i=0;
  while ((!feof(fp))&&(ch=fgetc(fp))&&(1+n-i)) {
    if ((((ch==' ')||(ch=='\n'))&&b)||((!(isalpha(ch)))&&(ch!='\n')&&(ch!=' ')))
      continue;
    else 
      if ((ch==' ')||(ch=='\n'))
	b++;
      else 
	b=0;
    t[i++]=ch;
  }
  t[n]='\0';
  fflush(stdin);
  fflush(fp);
  return n;
}

double EuklidDist(int n,int* ngram1,int* ngram2) {
  double d=0;
  int i;
  long int s1=0,s2=0;
  for (i=0;i<(pow2(27,n));i++) {
    s1+=ngram1[i];
    s2+=ngram2[i];
  }
  for (i=0;i<(pow2(27,n));i++) {
    d+=pow((double)ngram1[i]/(double)s1-(double)ngram2[i]/(double)s2,2);
  }
  return sqrt(d);
}

int main(int argc,char* argv[]) {
  int n;
  if (argc!=4) {
    printf("Usage: ngram <file1> <file2> <n>\n");
    return -1;
  }
  sscanf(argv[3],"%i",&n);
  buildNgram(argv[1],argv[2],n);
  printf("Distanz: %f\n",EuklidDist(n,ngram1,ngram2));
  return 1;
}

